package java.util.concurrent;

import java.security.ProtectionDomain;
import java.security.Permissions;
import java.security.AccessControlContext;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.Collections;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.util.Collection;
import java.util.Arrays;
import sun.misc.Unsafe;
import java.util.concurrent.atomic.AtomicLong;
import sun.misc.Contended;

@Contended
public class ForkJoinPool extends AbstractExecutorService
{
    static final int SMASK = 65535;
    static final int MAX_CAP = 32767;
    static final int EVENMASK = 65534;
    static final int SQMASK = 126;
    static final int SCANNING = 1;
    static final int INACTIVE = Integer.MIN_VALUE;
    static final int SS_SEQ = 65536;
    static final int MODE_MASK = -65536;
    static final int LIFO_QUEUE = 0;
    static final int FIFO_QUEUE = 65536;
    static final int SHARED_QUEUE = Integer.MIN_VALUE;
    public static final ForkJoinPool.ForkJoinWorkerThreadFactory defaultForkJoinWorkerThreadFactory;
    private static final RuntimePermission modifyThreadPermission;
    static final ForkJoinPool common;
    static final int commonParallelism;
    private static int commonMaxSpares;
    private static int poolNumberSequence;
    private static final long IDLE_TIMEOUT = 2000000000L;
    private static final long TIMEOUT_SLOP = 20000000L;
    private static final int DEFAULT_COMMON_MAX_SPARES = 256;
    private static final int SPINS = 0;
    private static final int SEED_INCREMENT = -1640531527;
    private static final long SP_MASK = 4294967295L;
    private static final long UC_MASK = -4294967296L;
    private static final int AC_SHIFT = 48;
    private static final long AC_UNIT = 281474976710656L;
    private static final long AC_MASK = -281474976710656L;
    private static final int TC_SHIFT = 32;
    private static final long TC_UNIT = 4294967296L;
    private static final long TC_MASK = 281470681743360L;
    private static final long ADD_WORKER = 140737488355328L;
    private static final int RSLOCK = 1;
    private static final int RSIGNAL = 2;
    private static final int STARTED = 4;
    private static final int STOP = 536870912;
    private static final int TERMINATED = 1073741824;
    private static final int SHUTDOWN = Integer.MIN_VALUE;
    volatile long ctl;
    volatile int runState;
    final int config;
    int indexSeed;
    volatile ForkJoinPool.WorkQueue[] workQueues;
    final ForkJoinPool.ForkJoinWorkerThreadFactory factory;
    final Thread.UncaughtExceptionHandler ueh;
    final String workerNamePrefix;
    volatile AtomicLong stealCounter;
    private static final Unsafe U;
    private static final int ABASE;
    private static final int ASHIFT;
    private static final long CTL;
    private static final long RUNSTATE;
    private static final long STEALCOUNTER;
    private static final long PARKBLOCKER;
    private static final long QTOP;
    private static final long QLOCK;
    private static final long QSCANSTATE;
    private static final long QPARKER;
    private static final long QCURRENTSTEAL;
    private static final long QCURRENTJOIN;
    
    private static void checkPermission() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(ForkJoinPool.modifyThreadPermission);
        }
    }
    
    private static final synchronized int nextPoolId() {
        return ++ForkJoinPool.poolNumberSequence;
    }
    
    private int lockRunState() {
        int runState;
        int n;
        return (((runState = this.runState) & 0x1) != 0x0 || !ForkJoinPool.U.compareAndSwapInt(this, ForkJoinPool.RUNSTATE, runState, n = (runState | 0x1))) ? this.awaitRunStateLock() : n;
    }
    
    private int awaitRunStateLock() {
        int n = 0;
        int n2 = 0;
        int nextSecondarySeed = 0;
        int n5;
        while (true) {
            int runState;
            if (((runState = this.runState) & 0x1) == 0x0) {
                if (ForkJoinPool.U.compareAndSwapInt(this, ForkJoinPool.RUNSTATE, runState, n5 = (runState | 0x1))) {
                    break;
                }
                continue;
            }
            else if (nextSecondarySeed == 0) {
                nextSecondarySeed = ThreadLocalRandom.nextSecondarySeed();
            }
            else if (n2 > 0) {
                int n3 = nextSecondarySeed ^ nextSecondarySeed << 6;
                int n4 = n3 ^ n3 >>> 21;
                nextSecondarySeed = (n4 ^ n4 << 7);
                if (nextSecondarySeed < 0) {
                    continue;
                }
                --n2;
            }
            else {
                AtomicLong stealCounter;
                if ((runState & 0x4) == 0x0 || (stealCounter = this.stealCounter) == null) {
                    Thread.yield();
                }
                else {
                    if (!ForkJoinPool.U.compareAndSwapInt(this, ForkJoinPool.RUNSTATE, runState, runState | 0x2)) {
                        continue;
                    }
                    synchronized (stealCounter) {
                        if ((this.runState & 0x2) != 0x0) {
                            try {
                                stealCounter.wait();
                            }
                            catch (InterruptedException ex) {
                                if (!(Thread.currentThread() instanceof ForkJoinWorkerThread)) {
                                    n = 1;
                                }
                                else {
                                    continue;
                                }
                            }
                        }
                        else {
                            stealCounter.notifyAll();
                        }
                    }
                }
            }
        }
        if (n != 0) {
            try {
                Thread.currentThread().interrupt();
            }
            catch (SecurityException ex2) {}
        }
        return n5;
    }
    
    private void unlockRunState(int expected, int n) {
        if (!ForkJoinPool.U.compareAndSwapInt(this, ForkJoinPool.RUNSTATE, expected, n)) {
            AtomicLong stealCounter = this.stealCounter;
            this.runState = n;
            if (stealCounter != null) {
                synchronized (stealCounter) {
                    stealCounter.notifyAll();
                }
            }
        }
    }
    
    private boolean createWorker() {
        ForkJoinPool.ForkJoinWorkerThreadFactory factory = this.factory;
        Throwable t = null;
        ForkJoinWorkerThread thread = null;
        try {
            if (factory != null && (thread = factory.newThread(this)) != null) {
                thread.start();
                return true;
            }
        }
        catch (Throwable t2) {
            t = t2;
        }
        this.deregisterWorker(thread, t);
        return false;
    }
    
    private void tryAddWorker(long ctl) {
        boolean compareAndSwapLong = false;
        do {
            long x = (0xFFFF000000000000L & ctl + 281474976710656L) | (0xFFFF00000000L & ctl + 4294967296L);
            if (this.ctl == ctl) {
                int lockRunState;
                int n;
                if ((n = ((lockRunState = this.lockRunState()) & 0x20000000)) == 0) {
                    compareAndSwapLong = ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, ctl, x);
                }
                this.unlockRunState(lockRunState, lockRunState & 0xFFFFFFFE);
                if (n != 0) {
                    break;
                }
                if (!(compareAndSwapLong == 0)) {
                    this.createWorker();
                    break;
                }
                continue;
            }
            else {
                continue;
            }
        } while (((ctl = this.ctl) & 0x800000000000L) != 0x0L && (int)ctl == 0);
    }
    
    final ForkJoinPool.WorkQueue registerWorker(ForkJoinWorkerThread forkJoinWorkerThread) {
        forkJoinWorkerThread.setDaemon(true);
        Thread.UncaughtExceptionHandler ueh;
        if ((ueh = this.ueh) != null) {
            forkJoinWorkerThread.setUncaughtExceptionHandler(ueh);
        }
        ForkJoinPool.WorkQueue workQueue = new ForkJoinPool.WorkQueue(this, forkJoinWorkerThread);
        int scanState = 0;
        int n = this.config & 0xFFFF0000;
        int lockRunState = this.lockRunState();
        try {
            ForkJoinPool.WorkQueue[] workQueues;
            int length;
            if ((workQueues = this.workQueues) != null && (length = workQueues.length) > 0) {
                int indexSeed = this.indexSeed - 1640531527;
                this.indexSeed = indexSeed;
                int hint = indexSeed;
                int n2 = length - 1;
                scanState = ((hint << 1 | 0x1) & n2);
                if (workQueues[scanState] != null) {
                    int n3 = 0;
                    while (workQueues[scanState = (scanState + ((length <= 4) ? 2 : ((length >>> 1 & 0xFFFE) + 2)) & n2)] != null) {
                        if (++n3 < length) {
                            continue;
                        }
                        workQueues = (this.workQueues = (ForkJoinPool.WorkQueue[])Arrays.copyOf(workQueues, length <<= 1));
                        n2 = length - 1;
                        n3 = 0;
                    }
                }
                workQueue.hint = hint;
                workQueue.config = (scanState | n);
                workQueues[workQueue.scanState = scanState] = workQueue;
            }
        }
        finally {
            this.unlockRunState(lockRunState, lockRunState & 0xFFFFFFFE);
        }
        forkJoinWorkerThread.setName(this.workerNamePrefix.concat(Integer.toString(scanState >>> 1)));
        return workQueue;
    }
    
    final void deregisterWorker(ForkJoinWorkerThread forkJoinWorkerThread, Throwable t) {
        ForkJoinPool.WorkQueue workQueue = null;
        if (forkJoinWorkerThread != null && (workQueue = forkJoinWorkerThread.workQueue) != null) {
            int n = workQueue.config & 0xFFFF;
            int lockRunState = this.lockRunState();
            ForkJoinPool.WorkQueue[] workQueues;
            if ((workQueues = this.workQueues) != null && workQueues.length > n && workQueues[n] == workQueue) {
                workQueues[n] = null;
            }
            this.unlockRunState(lockRunState, lockRunState & 0xFFFFFFFE);
        }
        long ctl;
        while (!ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, ctl = this.ctl, (0xFFFF000000000000L & ctl - 281474976710656L) | (0xFFFF00000000L & ctl - 4294967296L) | (0xFFFFFFFFL & ctl))) {}
        if (workQueue != null) {
            workQueue.qlock = -1;
            workQueue.transferStealCount(this);
            workQueue.cancelAll();
        }
        ForkJoinPool.WorkQueue[] workQueues2;
        while (!this.tryTerminate(false, false) && workQueue != null && workQueue.array != null && (this.runState & 0x20000000) == 0x0 && (workQueues2 = this.workQueues) != null) {
            int n2;
            if ((n2 = workQueues2.length - 1) < 0) {
                break;
            }
            long ctl2;
            int n3;
            if ((n3 = (int)(ctl2 = this.ctl)) != 0) {
                if (this.tryRelease(ctl2, workQueues2[n3 & n2], 281474976710656L)) {
                    break;
                }
                continue;
            }
            else {
                if (t != null && (ctl2 & 0x800000000000L) != 0x0L) {
                    this.tryAddWorker(ctl2);
                    break;
                }
                break;
            }
        }
        if (t == null) {
            ForkJoinTask.helpExpungeStaleExceptions();
        }
        else {
            ForkJoinTask.rethrow(t);
        }
    }
    
    final void signalWork(ForkJoinPool.WorkQueue[] array, ForkJoinPool.WorkQueue workQueue) {
        long ctl;
        while ((ctl = this.ctl) < 0L) {
            int n;
            if ((n = (int)ctl) == 0) {
                if ((ctl & 0x800000000000L) != 0x0L) {
                    this.tryAddWorker(ctl);
                    break;
                }
                break;
            }
            else {
                if (array == null) {
                    break;
                }
                int n2;
                if (array.length <= (n2 = (n & 0xFFFF))) {
                    break;
                }
                ForkJoinPool.WorkQueue workQueue2;
                if ((workQueue2 = array[n2]) == null) {
                    break;
                }
                int scanState = n + 65536 & Integer.MAX_VALUE;
                int n3 = n - workQueue2.scanState;
                long x = (0xFFFFFFFF00000000L & ctl + 281474976710656L) | (0xFFFFFFFFL & (long)workQueue2.stackPred);
                if (n3 == 0 && ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, ctl, x)) {
                    workQueue2.scanState = scanState;
                    Thread parker;
                    if ((parker = workQueue2.parker) != null) {
                        ForkJoinPool.U.unpark(parker);
                        break;
                    }
                    break;
                }
                else {
                    if (workQueue != null && workQueue.base == workQueue.top) {
                        break;
                    }
                    continue;
                }
            }
        }
    }
    
    private boolean tryRelease(long expected, ForkJoinPool.WorkQueue workQueue, long n) {
        int n2 = (int)expected;
        int scanState = n2 + 65536 & Integer.MAX_VALUE;
        if (workQueue != null && workQueue.scanState == n2 && ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, expected, (0xFFFFFFFF00000000L & expected + n) | (0xFFFFFFFFL & (long)workQueue.stackPred))) {
            workQueue.scanState = scanState;
            Thread parker;
            if ((parker = workQueue.parker) != null) {
                ForkJoinPool.U.unpark(parker);
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    final void runWorker(ForkJoinPool.WorkQueue workQueue) {
        workQueue.growArray();
        int hint = workQueue.hint;
        hint = ((hint == 0) ? 1 : hint);
        while (true) {
            ForkJoinTask<Object> scan;
            if ((scan = this.scan(workQueue, hint)) != null) {
                workQueue.runTask(scan);
            }
            else if (!this.awaitWork(workQueue, hint)) {
                break;
            }
            int n = hint ^ hint << 13;
            int n2 = n ^ n >>> 17;
            hint = (n2 ^ n2 << 5);
        }
    }
    
    private ForkJoinTask<?> scan(ForkJoinPool.WorkQueue o, int n) {
        ForkJoinPool.WorkQueue[] workQueues;
        int n2;
        if ((workQueues = this.workQueues) != null && (n2 = workQueues.length - 1) > 0 && o != null) {
            int scanState = o.scanState;
            int n3 = (n & n2);
            int n4 = 0;
            int n5 = 0;
            while (true) {
                ForkJoinPool.WorkQueue workQueue;
                if ((workQueue = workQueues[n3]) != null) {
                    int base;
                    int n6;
                    ForkJoinTask[] array;
                    if ((n6 = (base = workQueue.base) - workQueue.top) < 0 && (array = workQueue.array) != null) {
                        long n7 = (long)(((array.length - 1 & base) << ForkJoinPool.ASHIFT) + ForkJoinPool.ABASE);
                        ForkJoinTask<Object> expected;
                        if ((expected = (ForkJoinTask)ForkJoinPool.U.getObjectVolatile(array, n7)) != null && workQueue.base == base) {
                            if (scanState >= 0) {
                                if (ForkJoinPool.U.compareAndSwapObject(array, n7, expected, null)) {
                                    workQueue.base = base + 1;
                                    if (n6 < -1) {
                                        this.signalWork(workQueues, workQueue);
                                        return expected;
                                    }
                                    return expected;
                                }
                            }
                            else if (n4 == 0 && o.scanState < 0) {
                                long ctl = this.ctl;
                                this.tryRelease(ctl, workQueues[n2 & (int)ctl], 281474976710656L);
                            }
                        }
                        if (scanState < 0) {
                            scanState = o.scanState;
                        }
                        n ^= n << 1;
                        n ^= n >>> 3;
                        n ^= n << 10;
                        n3 = (n3 = (n & n2));
                        n5 = (n4 = 0);
                        continue;
                    }
                    else {
                        n5 += base;
                    }
                }
                if ((n3 = (n3 + 1 & n2)) == n3) {
                    if ((scanState >= 0 || scanState == (scanState = o.scanState)) && n4 == (n4 = n5)) {
                        if (scanState >= 0) {
                            if (o.qlock < 0) {
                                break;
                            }
                            int x = scanState | Integer.MIN_VALUE;
                            long ctl2;
                            long x2 = (0xFFFFFFFFL & (long)x) | (0xFFFFFFFF00000000L & (ctl2 = this.ctl) - 281474976710656L);
                            o.stackPred = (int)ctl2;
                            ForkJoinPool.U.putInt(o, ForkJoinPool.QSCANSTATE, x);
                            if (ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, ctl2, x2)) {
                                scanState = x;
                            }
                            else {
                                o.scanState = scanState;
                            }
                        }
                        else {
                            break;
                        }
                    }
                    n5 = 0;
                }
                else {
                    continue;
                }
            }
        }
        return null;
    }
    
    private boolean awaitWork(ForkJoinPool.WorkQueue o, int n) {
        if (o == null || o.qlock < 0) {
            return false;
        }
        int stackPred = o.stackPred;
        int n2 = 0;
        int scanState;
        while ((scanState = o.scanState) < 0) {
            if (n2 > 0) {
                n ^= n << 6;
                n ^= n >>> 21;
                n ^= n << 7;
                if (n < 0 || --n2 != 0) {
                    continue;
                }
                ForkJoinPool.WorkQueue[] workQueues;
                int n7;
                ForkJoinPool.WorkQueue workQueue;
                if (stackPred != 0 && (workQueues = this.workQueues) != null && (n7 = (stackPred & 0xFFFF)) < workQueues.length && (workQueue = workQueues[n7]) != null && (workQueue.parker == null || workQueue.scanState >= 0)) {
                    n2 = 0;
                }
                else {
                    continue;
                }
            }
            else {
                if (o.qlock < 0) {
                    return false;
                }
                if (Thread.interrupted()) {
                    continue;
                }
                long ctl;
                int n3 = (int)((ctl = this.ctl) >> 48) + (this.config & 0xFFFF);
                if ((n3 <= 0 && this.tryTerminate(false, false)) || (this.runState & 0x20000000) != 0x0) {
                    return false;
                }
                long n5;
                long n4;
                long time;
                if (n3 <= 0 && scanState == (int)ctl) {
                    n4 = ((0xFFFFFFFF00000000L & ctl + 281474976710656L) | (0xFFFFFFFFL & (long)stackPred));
                    int n6 = (short)(int)(ctl >>> 32);
                    if (n6 > 2 && ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, ctl, n4)) {
                        return false;
                    }
                    time = 2000000000L * (long)((n6 >= 0) ? 1 : (1 - n6));
                    n5 = System.nanoTime() + time - 20000000L;
                }
                else {
                    time = (n4 = (n5 = 0L));
                }
                Thread currentThread = Thread.currentThread();
                ForkJoinPool.U.putObject(currentThread, ForkJoinPool.PARKBLOCKER, this);
                o.parker = currentThread;
                if (o.scanState < 0 && this.ctl == ctl) {
                    ForkJoinPool.U.park(false, time);
                }
                ForkJoinPool.U.putOrderedObject(o, ForkJoinPool.QPARKER, null);
                ForkJoinPool.U.putObject(currentThread, ForkJoinPool.PARKBLOCKER, null);
                if (o.scanState >= 0) {
                    return true;
                }
                if (time != 0L && this.ctl == ctl && n5 - System.nanoTime() <= 0L && ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, ctl, n4)) {
                    return false;
                }
                continue;
            }
        }
        return true;
    }
    
    final int helpComplete(ForkJoinPool.WorkQueue workQueue, CountedCompleter<?> countedCompleter, int n) {
        int status = 0;
        ForkJoinPool.WorkQueue[] workQueues;
        int n2;
        if ((workQueues = this.workQueues) != null && (n2 = workQueues.length - 1) >= 0 && countedCompleter != null && workQueue != null) {
            int config = workQueue.config;
            int n3 = workQueue.hint ^ workQueue.top;
            int n4 = n3 & n2;
            int pollAndExecCC = 1;
            int n5 = 0;
            int n6 = 0;
            while ((status = countedCompleter.status) >= 0) {
                CountedCompleter<Object> popCC;
                if (pollAndExecCC == 1 && (popCC = workQueue.popCC(countedCompleter, config)) != null) {
                    popCC.doExec();
                    if (n != 0 && --n == 0) {
                        break;
                    }
                    n6 = (n5 = 0);
                }
                else {
                    ForkJoinPool.WorkQueue workQueue2;
                    if ((workQueue2 = workQueues[n4]) == null) {
                        pollAndExecCC = 0;
                    }
                    else if ((pollAndExecCC = workQueue2.pollAndExecCC(countedCompleter)) < 0) {
                        n6 += pollAndExecCC;
                    }
                    if (pollAndExecCC > 0) {
                        if (pollAndExecCC == 1 && n != 0 && --n == 0) {
                            break;
                        }
                        int n7 = n3 ^ n3 << 13;
                        int n8 = n7 ^ n7 >>> 17;
                        n3 = (n8 ^ n8 << 5);
                        n4 = (n4 = (n3 & n2));
                        n6 = (n5 = 0);
                    }
                    else if ((n4 = (n4 + 1 & n2)) == n4) {
                        if (n5 == (n5 = n6)) {
                            break;
                        }
                        n6 = 0;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        return status;
    }
    
    private void helpStealer(ForkJoinPool.WorkQueue p0, ForkJoinTask<?> p1) {
        // Bytecode:
        // 
        //     1: getfield        java/util/concurrent/ForkJoinPool.workQueues:[Ljava/util/concurrent/ForkJoinPool$WorkQueue;
        //     4: astore_3       
        //     5: iconst_0       
        //     6: istore          4
        //     8: aload_3        
        //     9: ifnull          393
        //    12: aload_3        
        //    13: arraylength    
        //    14: iconst_1       
        //    15: isub           
        //    16: dup            
        //    17: istore          6
        //    19: iflt            393
        //    22: aload_1        
        //    23: ifnull          393
        //    26: aload_2        
        //    27: ifnull          393
        //    30: iconst_0       
        //    31: istore          5
        //    33: aload_1        
        //    34: astore          8
        //    36: aload_2        
        //    37: astore          7
        //    39: aload           7
        //    41: getfield        java/util/concurrent/ForkJoinTask.status:I
        //    44: iflt            376
        //    47: aload           8
        //    49: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.hint:I
        //    52: iconst_1       
        //    53: ior            
        //    54: istore          10
        //    56: iconst_0       
        //    57: istore          11
        //    59: iload           11
        //    61: iload           6
        //    63: if_icmple       69
        //    66: goto            376
        //    69: aload_3        
        //    70: iload           10
        //    72: iload           11
        //    74: iadd           
        //    75: iload           6
        //    77: iand           
        //    78: dup            
        //    79: istore          12
        //    81: aaload         
        //    82: dup            
        //    83: astore          9
        //    85: ifnull          118
        //    88: aload           9
        //    90: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.currentSteal:Ljava/util/concurrent/ForkJoinTask;
        //    93: aload           7
        //    95: if_acmpne       108
        //    98: aload           8
        //   100: iload           12
        //   102: putfield        java/util/concurrent/ForkJoinPool$WorkQueue.hint:I
        //   105: goto            124
        //   108: iload           5
        //   110: aload           9
        //   112: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.base:I
        //   115: iadd           
        //   116: istore          5
        //   118: iinc            11, 2
        //   121: goto            59
        //   124: iload           5
        //   126: aload           9
        //   128: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.base:I
        //   131: dup            
        //   132: istore          11
        //   134: iadd           
        //   135: istore          5
        //   137: aload           9
        //   139: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.currentJoin:Ljava/util/concurrent/ForkJoinTask;
        //   142: astore          12
        //   144: aload           7
        //   146: getfield        java/util/concurrent/ForkJoinTask.status:I
        //   149: iflt            376
        //   152: aload           8
        //   154: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.currentJoin:Ljava/util/concurrent/ForkJoinTask;
        //   157: aload           7
        //   159: if_acmpne       376
        //   162: aload           9
        //   164: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.currentSteal:Ljava/util/concurrent/ForkJoinTask;
        //   167: aload           7
        //   169: if_acmpeq       175
        //   172: goto            376
        //   175: iload           11
        //   177: aload           9
        //   179: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.top:I
        //   182: isub           
        //   183: ifge            197
        //   186: aload           9
        //   188: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.array:[Ljava/util/concurrent/ForkJoinTask;
        //   191: dup            
        //   192: astore          10
        //   194: ifnonnull       215
        //   197: aload           12
        //   199: dup            
        //   200: astore          7
        //   202: ifnonnull       208
        //   205: goto            376
        //   208: aload           9
        //   210: astore          8
        //   212: goto            39
        //   215: aload           10
        //   217: arraylength    
        //   218: iconst_1       
        //   219: isub           
        //   220: iload           11
        //   222: iand           
        //   223: getstatic       java/util/concurrent/ForkJoinPool.ASHIFT:I
        //   226: ishl           
        //   227: getstatic       java/util/concurrent/ForkJoinPool.ABASE:I
        //   230: iadd           
        //   231: istore          13
        //   233: getstatic       java/util/concurrent/ForkJoinPool.U:Lsun/misc/Unsafe;
        //   236: aload           10
        //   238: iload           13
        //   240: i2l            
        //   241: invokevirtual   sun/misc/Unsafe.getObjectVolatile:(Ljava/lang/Object;J)Ljava/lang/Object;
        //   244: checkcast       Ljava/util/concurrent/ForkJoinTask;
        //   247: astore          14
        //   249: aload           9
        //   251: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.base:I
        //   254: iload           11
        //   256: if_icmpne       373
        //   259: aload           14
        //   261: ifnonnull       267
        //   264: goto            376
        //   267: getstatic       java/util/concurrent/ForkJoinPool.U:Lsun/misc/Unsafe;
        //   270: aload           10
        //   272: iload           13
        //   274: i2l            
        //   275: aload           14
        //   277: aconst_null    
        //   278: invokevirtual   sun/misc/Unsafe.compareAndSwapObject:(Ljava/lang/Object;JLjava/lang/Object;Ljava/lang/Object;)Z
        //   281: ifeq            373
        //   284: aload           9
        //   286: iload           11
        //   288: iconst_1       
        //   289: iadd           
        //   290: putfield        java/util/concurrent/ForkJoinPool$WorkQueue.base:I
        //   293: aload_1        
        //   294: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.currentSteal:Ljava/util/concurrent/ForkJoinTask;
        //   297: astore          15
        //   299: aload_1        
        //   300: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.top:I
        //   303: istore          16
        //   305: getstatic       java/util/concurrent/ForkJoinPool.U:Lsun/misc/Unsafe;
        //   308: aload_1        
        //   309: getstatic       java/util/concurrent/ForkJoinPool.QCURRENTSTEAL:J
        //   312: aload           14
        //   314: invokevirtual   sun/misc/Unsafe.putOrderedObject:(Ljava/lang/Object;JLjava/lang/Object;)V
        //   317: aload           14
        //   319: invokevirtual   java/util/concurrent/ForkJoinTask.doExec:()I
        //   322: pop            
        //   323: aload_2        
        //   324: getfield        java/util/concurrent/ForkJoinTask.status:I
        //   327: iflt            349
        //   330: aload_1        
        //   331: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.top:I
        //   334: iload           16
        //   336: if_icmpeq       349
        //   339: aload_1        
        //   340: invokevirtual   java/util/concurrent/ForkJoinPool$WorkQueue.pop:()Ljava/util/concurrent/ForkJoinTask;
        //   343: dup            
        //   344: astore          14
        //   346: ifnonnull       305
        //   349: getstatic       java/util/concurrent/ForkJoinPool.U:Lsun/misc/Unsafe;
        //   352: aload_1        
        //   353: getstatic       java/util/concurrent/ForkJoinPool.QCURRENTSTEAL:J
        //   356: aload           15
        //   358: invokevirtual   sun/misc/Unsafe.putOrderedObject:(Ljava/lang/Object;JLjava/lang/Object;)V
        //   361: aload_1        
        //   362: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.base:I
        //   365: aload_1        
        //   366: getfield        java/util/concurrent/ForkJoinPool$WorkQueue.top:I
        //   369: if_icmpeq       373
        //   372: return         
        //   373: goto            124
        //   376: aload_2        
        //   377: getfield        java/util/concurrent/ForkJoinTask.status:I
        //   380: iflt            393
        //   383: iload           4
        //   385: iload           5
        //   387: dup            
        //   388: istore          4
        //   390: if_icmpne       30
        //   393: return         
        //    Signature:
        //  (Ljava/util/concurrent/ForkJoinPool$WorkQueue;Ljava/util/concurrent/ForkJoinTask<*>;)V
        //    StackMapTable: 00 11 FF 00 1E 00 07 07 00 1E 07 00 1F 07 01 C0 07 01 99 01 00 01 00 00 FF 00 08 00 09 07 00 1E 07 00 1F 07 01 C0 07 01 99 01 01 01 07 01 C0 07 00 1F 00 00 FE 00 13 00 01 01 09 FF 00 26 00 0D 07 00 1E 07 00 1F 07 01 C0 07 01 99 01 01 01 07 01 C0 07 00 1F 07 00 1F 01 01 01 00 00 09 F8 00 05 FE 00 32 00 01 07 01 C0 15 0A FF 00 06 00 0D 07 00 1E 07 00 1F 07 01 C0 07 01 99 01 01 01 07 01 C0 07 00 1F 07 00 1F 07 01 9A 01 07 01 C0 00 00 FD 00 33 01 07 01 C0 FD 00 25 07 01 C0 01 2B FF 00 17 00 0A 07 00 1E 07 00 1F 07 01 C0 07 01 99 01 01 01 07 01 C0 07 00 1F 07 00 1F 00 00 F8 00 02 F9 00 10
        // 
        throw new IllegalStateException("An error occurred while decompiling this method.");
    }
    
    private boolean tryCompensate(ForkJoinPool.WorkQueue workQueue) {
        ForkJoinPool.WorkQueue[] workQueues;
        int n;
        int n2;
        int n7;
        if (workQueue == null || workQueue.qlock < 0 || (workQueues = this.workQueues) == null || (n = workQueues.length - 1) <= 0 || (n2 = (this.config & 0xFFFF)) == 0) {
            n7 = false;
        }
        else {
            long ctl;
            int n3;
            if ((n3 = (int)(ctl = this.ctl)) != 0) {
                n7 = this.tryRelease(ctl, workQueues[n3 & n], 0L);
            }
            else {
                int n4 = (int)(ctl >> 48) + n2;
                int n5 = (short)(int)(ctl >> 32) + n2;
                int n6 = 0;
                for (int i = 0; i <= n; ++i) {
                    ForkJoinPool.WorkQueue workQueue2;
                    if ((workQueue2 = workQueues[(i << 1 | 0x1) & n]) != null) {
                        if ((workQueue2.scanState & 0x1) != 0x0) {
                            break;
                        }
                        ++n6;
                    }
                }
                if (n6 != n5 << 1 || this.ctl != ctl) {
                    n7 = false;
                }
                else if (n5 >= n2 && n4 > 1 && workQueue.isEmpty()) {
                    n7 = ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, ctl, (0xFFFF000000000000L & ctl - 281474976710656L) | (0xFFFFFFFFFFFFL & ctl));
                }
                else {
                    if (n5 >= 32767 || (this == ForkJoinPool.common && n5 >= n2 + ForkJoinPool.commonMaxSpares)) {
                        throw new RejectedExecutionException("Thread limit exceeded replacing blocked worker");
                    }
                    boolean compareAndSwapLong = false;
                    long x = (0xFFFF000000000000L & ctl) | (0xFFFF00000000L & ctl + 4294967296L);
                    int lockRunState;
                    if (((lockRunState = this.lockRunState()) & 0x20000000) == 0x0) {
                        compareAndSwapLong = ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, ctl, x);
                    }
                    this.unlockRunState(lockRunState, lockRunState & 0xFFFFFFFE);
                    n7 = (compareAndSwapLong && this.createWorker());
                }
            }
        }
        return n7;
    }
    
    final int awaitJoin(ForkJoinPool.WorkQueue workQueue, ForkJoinTask<?> x, long n) {
        int n2 = 0;
        if (x != null && workQueue != null) {
            ForkJoinTask<Object> currentJoin = workQueue.currentJoin;
            ForkJoinPool.U.putOrderedObject(workQueue, ForkJoinPool.QCURRENTJOIN, x);
            CountedCompleter<Object> countedCompleter = (x instanceof CountedCompleter) ? ((CountedCompleter)x) : null;
            Label_0191: {
                while ((n2 = x.status) >= 0) {
                    if (countedCompleter != null) {
                        this.helpComplete(workQueue, countedCompleter, 0);
                    }
                    else if (workQueue.base == workQueue.top || workQueue.tryRemoveAndExec(x)) {
                        this.helpStealer(workQueue, x);
                    }
                    if ((n2 = x.status) >= 0) {
                        long millis;
                        if (n == 0L) {
                            millis = 0L;
                        }
                        else {
                            long n3;
                            if ((n3 = n - System.nanoTime()) <= 0L) {
                                break;
                            }
                            if ((millis = TimeUnit.NANOSECONDS.toMillis(n3)) <= 0L) {
                                millis = 1L;
                            }
                        }
                        if (this.tryCompensate(workQueue)) {
                            x.internalWait(millis);
                            ForkJoinPool.U.getAndAddLong(this, ForkJoinPool.CTL, 281474976710656L);
                        }
                        else {
                            continue;
                        }
                        continue;
                    }
                    ForkJoinPool.U.putOrderedObject(workQueue, ForkJoinPool.QCURRENTJOIN, currentJoin);
                    return n2;
                }
                continue;
            }
        }
        return n2;
    }
    
    private ForkJoinPool.WorkQueue findNonEmptyStealQueue() {
        int nextSecondarySeed = ThreadLocalRandom.nextSecondarySeed();
        ForkJoinPool.WorkQueue[] workQueues;
        int n;
        if ((workQueues = this.workQueues) != null && (n = workQueues.length - 1) >= 0) {
            int n2 = (nextSecondarySeed & n);
            int n3 = 0;
            int n4 = 0;
            while (true) {
                ForkJoinPool.WorkQueue workQueue;
                if ((workQueue = workQueues[n2]) != null) {
                    int base;
                    if ((base = workQueue.base) - workQueue.top < 0) {
                        return workQueue;
                    }
                    n4 += base;
                }
                if ((n2 = (n2 + 1 & n)) == n2) {
                    if (n3 == (n3 = n4)) {
                        break;
                    }
                    n4 = 0;
                }
                else {
                    continue;
                }
            }
        }
        return null;
    }
    
    final void helpQuiescePool(ForkJoinPool.WorkQueue workQueue) {
        ForkJoinTask<Object> currentSteal = workQueue.currentSteal;
        int n = 1;
        while (true) {
            workQueue.execLocalTasks();
            ForkJoinPool.WorkQueue nonEmptyStealQueue;
            if ((nonEmptyStealQueue = this.findNonEmptyStealQueue()) != null) {
                if (n == 0) {
                    n = 1;
                    ForkJoinPool.U.getAndAddLong(this, ForkJoinPool.CTL, 281474976710656L);
                }
                int base;
                ForkJoinTask<Object> poll;
                if ((base = nonEmptyStealQueue.base) - nonEmptyStealQueue.top < 0 && (poll = nonEmptyStealQueue.pollAt(base)) != null) {
                    ForkJoinPool.U.putOrderedObject(workQueue, ForkJoinPool.QCURRENTSTEAL, poll);
                    poll.doExec();
                    if (++workQueue.nsteals < 0) {
                        workQueue.transferStealCount(this);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
            else if (n != 0) {
                long ctl2;
                long x = (0xFFFF000000000000L & (ctl2 = this.ctl) - 281474976710656L) | (0xFFFFFFFFFFFFL & ctl2);
                if ((int)(x >> 48) + (this.config & 0xFFFF) <= 0) {
                    break;
                }
                if (ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, ctl2, x)) {
                    n = 0;
                }
                else {
                    continue;
                }
            }
            else {
                long ctl;
                if ((int)((ctl = this.ctl) >> 48) + (this.config & 0xFFFF) <= 0 && ForkJoinPool.U.compareAndSwapLong(this, ForkJoinPool.CTL, ctl, ctl + 281474976710656L)) {
                    break;
                }
                continue;
            }
        }
        ForkJoinPool.U.putOrderedObject(workQueue, ForkJoinPool.QCURRENTSTEAL, currentSteal);
    }
    
    final ForkJoinTask<?> nextTaskFor(ForkJoinPool.WorkQueue workQueue) {
        ForkJoinTask<Object> nextLocalTask;
        while ((nextLocalTask = workQueue.nextLocalTask()) == null) {
            ForkJoinPool.WorkQueue nonEmptyStealQueue;
            if ((nonEmptyStealQueue = this.findNonEmptyStealQueue()) == null) {
                return null;
            }
            int base;
            ForkJoinTask<Object> poll;
            if ((base = nonEmptyStealQueue.base) - nonEmptyStealQueue.top < 0 && (poll = nonEmptyStealQueue.pollAt(base)) != null) {
                return poll;
            }
            continue;
        }
        return nextLocalTask;
    }
    
    static int getSurplusQueuedTaskCount() {
        Thread currentThread;
        if (((currentThread = Thread.currentThread()) instanceof ForkJoinWorkerThread)) {
            ForkJoinWorkerThread forkJoinWorkerThread;
            ForkJoinPool pool;
            int n = (pool = (forkJoinWorkerThread = (ForkJoinWorkerThread)currentThread).pool).config & 0xFFFF;
            ForkJoinPool.WorkQueue workQueue = forkJoinWorkerThread.workQueue;
            int n2 = workQueue.top - workQueue.base;
            int n3 = (int)(pool.ctl >> 48) + n;
            int n4;
            int n5;
            int n6;
            return n2 - ((n3 > (n4 = n >>> 1)) ? 0 : ((n3 > (n5 = n4 >>> 1)) ? 1 : ((n3 > (n6 = n5 >>> 1)) ? 2 : ((n3 > n6 >>> 1) ? 4 : 8))));
        }
        return 0;
    }
    
    private boolean tryTerminate(boolean b, boolean b2) {
        if (this == ForkJoinPool.common) {
            return false;
        }
        int n;
        if ((n = this.runState) >= 0) {
            if (!b2) {
                return false;
            }
            n = this.lockRunState();
            this.unlockRunState(n, (n & 0xFFFFFFFE) | Integer.MIN_VALUE);
        }
        if ((n & 0x20000000) == 0x0) {
            if (!b) {
                long n2 = 0L;
                while (true) {
                    long ctl = this.ctl;
                    if ((int)(ctl >> 48) + (this.config & 0xFFFF) > 0) {
                        return false;
                    }
                    ForkJoinPool.WorkQueue[] workQueues;
                    if ((workQueues = this.workQueues) != null) {
                        int n3;
                        if ((n3 = workQueues.length - 1) <= 0) {
                            break;
                        }
                        for (int i = 0; i <= n3; ++i) {
                            ForkJoinPool.WorkQueue workQueue;
                            if ((workQueue = workQueues[i]) != null) {
                                int base;
                                if ((base = workQueue.base) != workQueue.top || workQueue.scanState >= 0 || workQueue.currentSteal != null) {
                                    long ctl2 = this.ctl;
                                    this.tryRelease(ctl2, workQueues[n3 & (int)ctl2], 281474976710656L);
                                    return false;
                                }
                                ctl += (long)base;
                                if ((i & 0x1) == 0x0) {
                                    workQueue.qlock = -1;
                                }
                            }
                        }
                        if (n2 == (n2 = ctl)) {
                            break;
                        }
                        continue;
                    }
                    else {
                        break;
                    }
                }
            }
            if ((this.runState & 0x20000000) == 0x0) {
                int lockRunState = this.lockRunState();
                this.unlockRunState(lockRunState, (lockRunState & 0xFFFFFFFE) | 0x20000000);
            }
        }
        int n4 = 0;
        long n5 = 0L;
        while (true) {
            long ctl3 = this.ctl;
            ForkJoinPool.WorkQueue[] workQueues2;
            int n6;
            if ((short)(int)(ctl3 >>> 32) + (this.config & 0xFFFF) <= 0 || (workQueues2 = this.workQueues) == null || (n6 = workQueues2.length - 1) <= 0) {
                if ((this.runState & 0x40000000) == 0x0) {
                    int lockRunState2 = this.lockRunState();
                    this.unlockRunState(lockRunState2, (lockRunState2 & 0xFFFFFFFE) | 0x40000000);
                    synchronized (this) {
                        this.notifyAll();
                    }
                    break;
                }
                break;
            }
            else {
                for (int j = 0; j <= n6; ++j) {
                    ForkJoinPool.WorkQueue workQueue2;
                    if ((workQueue2 = workQueues2[j]) != null) {
                        ctl3 += (long)workQueue2.base;
                        workQueue2.qlock = -1;
                        if (n4 > 0) {
                            workQueue2.cancelAll();
                            ForkJoinWorkerThread owner;
                            if (n4 > 1 && (owner = workQueue2.owner) != null) {
                                if (!owner.isInterrupted()) {
                                    try {
                                        owner.interrupt();
                                    }
                                    catch (Throwable t) {}
                                }
                                if (workQueue2.scanState < 0) {
                                    ForkJoinPool.U.unpark(owner);
                                }
                            }
                        }
                    }
                }
                if (ctl3 != n5) {
                    n5 = ctl3;
                    n4 = 0;
                }
                else {
                    if (n4 > 3 && n4 > n6) {
                        break;
                    }
                    if (++n4 > 1) {
                        int n7 = 0;
                        long ctl4;
                        int n8;
                        while (n7++ <= n6 && (n8 = (int)(ctl4 = this.ctl)) != 0) {
                            this.tryRelease(ctl4, workQueues2[n8 & n6], 281474976710656L);
                        }
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        return true;
    }
    
    private void externalSubmit(ForkJoinTask<?> x) {
        int hint;
        if ((hint = ThreadLocalRandom.getProbe()) == 0) {
            ThreadLocalRandom.localInit();
            hint = ThreadLocalRandom.getProbe();
        }
        while (true) {
            boolean n = false;
            int runState;
            if ((runState = this.runState) < 0) {
                this.tryTerminate(false, false);
                throw new RejectedExecutionException();
            }
            ForkJoinPool.WorkQueue[] workQueues;
            int n2;
            if ((runState & 0x4) == 0x0 || (workQueues = this.workQueues) == null || (n2 = workQueues.length - 1) < 0) {
                int n5 = 0;
                int lockRunState2 = this.lockRunState();
                try {
                    if ((lockRunState2 & 0x4) == 0x0) {
                        ForkJoinPool.U.compareAndSwapObject(this, ForkJoinPool.STEALCOUNTER, null, new AtomicLong());
                        int n6 = this.config & 0xFFFF;
                        int n7 = (n6 > 1) ? (n6 - 1) : 1;
                        int n8 = n7 | n7 >>> 1;
                        int n9 = n8 | n8 >>> 2;
                        int n10 = n9 | n9 >>> 4;
                        int n11 = n10 | n10 >>> 8;
                        this.workQueues = new ForkJoinPool.WorkQueue[(n11 | n11 >>> 16) + 1 << 1];
                        n5 = 4;
                    }
                }
                finally {
                    this.unlockRunState(lockRunState2, (lockRunState2 & 0xFFFFFFFE) | n5);
                }
            }
            else {
                int n3;
                ForkJoinPool.WorkQueue o;
                if ((o = workQueues[n3 = (hint & n2 & 0x7E)]) != null) {
                    if (o.qlock == 0 && ForkJoinPool.U.compareAndSwapInt(o, ForkJoinPool.QLOCK, 0, 1)) {
                        ForkJoinTask[] o3 = o.array;
                        int top = o.top;
                        boolean n4 = false;
                        try {
                            if ((o3 != null && o3.length > top + 1 - o.base) || (o3 = o.growArray()) != null) {
                                ForkJoinPool.U.putOrderedObject(o3, (long)(((o3.length - 1 & top) << ForkJoinPool.ASHIFT) + ForkJoinPool.ABASE), x);
                                ForkJoinPool.U.putOrderedInt(o, ForkJoinPool.QTOP, top + 1);
                                n4 = true;
                            }
                        }
                        finally {
                            ForkJoinPool.U.compareAndSwapInt(o, ForkJoinPool.QLOCK, 1, 0);
                        }
                        if (!(n4 == 0)) {
                            this.signalWork(workQueues, o);
                            return;
                        }
                    }
                    n = true;
                }
                else if ((this.runState & 0x1) == 0x0) {
                    ForkJoinPool.WorkQueue o2 = new ForkJoinPool.WorkQueue(this, null);
                    o2.hint = hint;
                    o2.config = (n3 | Integer.MIN_VALUE);
                    o2.scanState = Integer.MIN_VALUE;
                    int lockRunState = this.lockRunState();
                    ForkJoinPool.WorkQueue[] workQueues2;
                    if (lockRunState > 0 && (workQueues2 = this.workQueues) != null && n3 < workQueues2.length && workQueues2[n3] == null) {
                        workQueues2[n3] = o2;
                    }
                    this.unlockRunState(lockRunState, lockRunState & 0xFFFFFFFE);
                }
                else {
                    n = true;
                }
            }
            if (!(n == 0)) {
                hint = ThreadLocalRandom.advanceProbe(hint);
            }
            else {
                continue;
            }
        }
    }
    
    final void externalPush(ForkJoinTask<?> x) {
        int probe = ThreadLocalRandom.getProbe();
        int runState = this.runState;
        ForkJoinPool.WorkQueue[] workQueues;
        int n;
        ForkJoinPool.WorkQueue workQueue;
        if ((workQueues = this.workQueues) != null && (n = workQueues.length - 1) >= 0 && (workQueue = workQueues[n & probe & 0x7E]) != null && probe != 0 && runState > 0 && ForkJoinPool.U.compareAndSwapInt(workQueue, ForkJoinPool.QLOCK, 0, 1)) {
            ForkJoinTask[] array;
            if ((array = workQueue.array) != null) {
                int n2 = array.length - 1;
                int top;
                int n3;
                if (n2 > (n3 = (top = workQueue.top) - workQueue.base)) {
                    ForkJoinPool.U.putOrderedObject(array, (long)(((n2 & top) << ForkJoinPool.ASHIFT) + ForkJoinPool.ABASE), x);
                    ForkJoinPool.U.putOrderedInt(workQueue, ForkJoinPool.QTOP, top + 1);
                    ForkJoinPool.U.putIntVolatile(workQueue, ForkJoinPool.QLOCK, 0);
                    if (n3 <= 1) {
                        this.signalWork(workQueues, workQueue);
                    }
                    return;
                }
            }
            ForkJoinPool.U.compareAndSwapInt(workQueue, ForkJoinPool.QLOCK, 1, 0);
        }
        this.externalSubmit(x);
    }
    
    static ForkJoinPool.WorkQueue commonSubmitterQueue() {
        ForkJoinPool common = ForkJoinPool.common;
        int probe = ThreadLocalRandom.getProbe();
        ForkJoinPool.WorkQueue[] workQueues;
        int n;
        return (common != null && (workQueues = common.workQueues) != null && (n = workQueues.length - 1) >= 0) ? workQueues[n & probe & 0x7E] : null;
    }
    
    final boolean tryExternalUnpush(ForkJoinTask<?> expected) {
        int probe = ThreadLocalRandom.getProbe();
        ForkJoinPool.WorkQueue[] workQueues;
        int n;
        ForkJoinPool.WorkQueue workQueue;
        ForkJoinTask[] array;
        int top;
        if ((workQueues = this.workQueues) != null && (n = workQueues.length - 1) >= 0 && (workQueue = workQueues[n & probe & 0x7E]) != null && (array = workQueue.array) != null && (top = workQueue.top) != workQueue.base) {
            long n2 = (long)(((array.length - 1 & top - 1) << ForkJoinPool.ASHIFT) + ForkJoinPool.ABASE);
            if (ForkJoinPool.U.compareAndSwapInt(workQueue, ForkJoinPool.QLOCK, 0, 1)) {
                if (workQueue.top == top && workQueue.array == array && ForkJoinPool.U.getObject(array, n2) == expected && ForkJoinPool.U.compareAndSwapObject(array, n2, expected, null)) {
                    ForkJoinPool.U.putOrderedInt(workQueue, ForkJoinPool.QTOP, top - 1);
                    ForkJoinPool.U.putOrderedInt(workQueue, ForkJoinPool.QLOCK, 0);
                    return true;
                }
                ForkJoinPool.U.compareAndSwapInt(workQueue, ForkJoinPool.QLOCK, 1, 0);
            }
        }
        return false;
    }
    
    final int externalHelpComplete(CountedCompleter<?> countedCompleter, int n) {
        int probe = ThreadLocalRandom.getProbe();
        ForkJoinPool.WorkQueue[] workQueues;
        int length;
        return ((workQueues = this.workQueues) == null || (length = workQueues.length) == 0) ? 0 : this.helpComplete(workQueues[length - 1 & probe & 0x7E], countedCompleter, n);
    }
    
    public ForkJoinPool() {
        this(Math.min(32767, Runtime.getRuntime().availableProcessors()), ForkJoinPool.defaultForkJoinWorkerThreadFactory, null, false);
    }
    
    public ForkJoinPool(int n) {
        this(n, ForkJoinPool.defaultForkJoinWorkerThreadFactory, null, false);
    }
    
    public ForkJoinPool(int n, ForkJoinPool.ForkJoinWorkerThreadFactory forkJoinWorkerThreadFactory, Thread.UncaughtExceptionHandler uncaughtExceptionHandler, boolean b) {
        this(checkParallelism(n), checkFactory(forkJoinWorkerThreadFactory), uncaughtExceptionHandler, b ? 65536 : 0, "ForkJoinPool-" + nextPoolId() + "-worker-");
        checkPermission();
    }
    
    private static int checkParallelism(int n) {
        if (n <= 0 || n > 32767) {
            throw new IllegalArgumentException();
        }
        return n;
    }
    
    private static ForkJoinPool.ForkJoinWorkerThreadFactory checkFactory(ForkJoinPool.ForkJoinWorkerThreadFactory forkJoinWorkerThreadFactory) {
        if (forkJoinWorkerThreadFactory == null) {
            throw new NullPointerException();
        }
        return forkJoinWorkerThreadFactory;
    }
    
    private ForkJoinPool(int n, ForkJoinPool.ForkJoinWorkerThreadFactory factory, Thread.UncaughtExceptionHandler ueh, int n2, String workerNamePrefix) {
        this.workerNamePrefix = workerNamePrefix;
        this.factory = factory;
        this.ueh = ueh;
        this.config = ((n & 0xFFFF) | n2);
        long n3 = (long)(-n);
        this.ctl = ((n3 << 48 & 0xFFFF000000000000L) | (n3 << 32 & 0xFFFF00000000L));
    }
    
    public static ForkJoinPool commonPool() {
        return ForkJoinPool.common;
    }
    
    public <T> T invoke(ForkJoinTask<T> forkJoinTask) {
        if (forkJoinTask == null) {
            throw new NullPointerException();
        }
        this.externalPush(forkJoinTask);
        return forkJoinTask.join();
    }
    
    public void execute(ForkJoinTask<?> forkJoinTask) {
        if (forkJoinTask == null) {
            throw new NullPointerException();
        }
        this.externalPush(forkJoinTask);
        return;
    }
    
    @Override
    public void execute(Runnable runnable) {
        if (runnable == null) {
            throw new NullPointerException();
        }
        ForkJoinTask<Object> forkJoinTask;
        if ((runnable instanceof ForkJoinTask)) {
            forkJoinTask = (ForkJoinTask)runnable;
        }
        else {
            forkJoinTask = new ForkJoinTask.RunnableExecuteAction(runnable);
        }
        this.externalPush(forkJoinTask);
        return;
    }
    
    public <T> ForkJoinTask<T> submit(ForkJoinTask<T> forkJoinTask) {
        if (forkJoinTask == null) {
            throw new NullPointerException();
        }
        this.externalPush(forkJoinTask);
        return forkJoinTask;
    }
    
    @Override
    public <T> ForkJoinTask<T> submit(Callable<T> callable) {
        ForkJoinTask.AdaptedCallable adaptedCallable = new ForkJoinTask.AdaptedCallable(callable);
        this.externalPush(adaptedCallable);
        return adaptedCallable;
    }
    
    @Override
    public <T> ForkJoinTask<T> submit(Runnable runnable, T t) {
        ForkJoinTask.AdaptedRunnable adaptedRunnable = new ForkJoinTask.AdaptedRunnable(runnable, t);
        this.externalPush(adaptedRunnable);
        return adaptedRunnable;
    }
    
    @Override
    public ForkJoinTask<?> submit(Runnable runnable) {
        if (runnable == null) {
            throw new NullPointerException();
        }
        ForkJoinTask<Object> forkJoinTask;
        if ((runnable instanceof ForkJoinTask)) {
            forkJoinTask = (ForkJoinTask)runnable;
        }
        else {
            forkJoinTask = new ForkJoinTask.AdaptedRunnableAction(runnable);
        }
        this.externalPush(forkJoinTask);
        return forkJoinTask;
    }
    
    @Override
    public <T> List<Future<T>> invokeAll(Collection<? extends Callable<T>> collection) {
        ArrayList list = new ArrayList(collection.size());
        boolean n = false;
        try {
            Iterator<Object> iterator = collection.iterator();
            while (iterator.hasNext()) {
                ForkJoinTask.AdaptedCallable e = new ForkJoinTask.AdaptedCallable((Callable)iterator.next());
                list.add(e);
                this.externalPush(e);
            }
            for (int i = 0; i < list.size(); ++i) {
                ((ForkJoinTask)list.get(i)).quietlyJoin();
            }
            n = true;
            if (n == 0) {
                for (int j = 0; j < list.size(); ++j) {
                    ((Future)list.get(j)).cancel(false);
                }
                return list;
            }
            return list;
        }
        finally {
            if (n == 0) {
                for (int k = 0; k < list.size(); ++k) {
                    ((Future)list.get(k)).cancel(false);
                }
            }
        }
    }
    
    public ForkJoinPool.ForkJoinWorkerThreadFactory getFactory() {
        return this.factory;
    }
    
    public Thread.UncaughtExceptionHandler getUncaughtExceptionHandler() {
        return this.ueh;
    }
    
    public int getParallelism() {
        int n;
        return ((n = (this.config & 0xFFFF)) > 0) ? n : true;
    }
    
    public static int getCommonPoolParallelism() {
        return ForkJoinPool.commonParallelism;
    }
    
    public int getPoolSize() {
        return (this.config & 0xFFFF) + (short)(int)(this.ctl >>> 32);
    }
    
    public boolean getAsyncMode() {
        return (this.config & 0x10000) != 0x0;
    }
    
    public int getRunningThreadCount() {
        int n = 0;
        ForkJoinPool.WorkQueue[] workQueues;
        if ((workQueues = this.workQueues) != null) {
            for (int i = 1; i < workQueues.length; i += 2) {
                ForkJoinPool.WorkQueue workQueue;
                if ((workQueue = workQueues[i]) != null && workQueue.isApparentlyUnblocked()) {
                    ++n;
                }
            }
        }
        return n;
    }
    
    public int getActiveThreadCount() {
        int n = (this.config & 0xFFFF) + (int)(this.ctl >> 48);
        return (n <= 0) ? 0 : n;
    }
    
    public boolean isQuiescent() {
        return (this.config & 0xFFFF) + (int)(this.ctl >> 48) <= 0;
    }
    
    public long getStealCount() {
        AtomicLong stealCounter = this.stealCounter;
        long n = (stealCounter == null) ? 0L : stealCounter.get();
        ForkJoinPool.WorkQueue[] workQueues;
        if ((workQueues = this.workQueues) != null) {
            for (int i = 1; i < workQueues.length; i += 2) {
                ForkJoinPool.WorkQueue workQueue;
                if ((workQueue = workQueues[i]) != null) {
                    n += (long)workQueue.nsteals;
                }
            }
        }
        return n;
    }
    
    public long getQueuedTaskCount() {
        long n = 0L;
        ForkJoinPool.WorkQueue[] workQueues;
        if ((workQueues = this.workQueues) != null) {
            for (int i = 1; i < workQueues.length; i += 2) {
                ForkJoinPool.WorkQueue workQueue;
                if ((workQueue = workQueues[i]) != null) {
                    n += (long)workQueue.queueSize();
                }
            }
        }
        return n;
    }
    
    public int getQueuedSubmissionCount() {
        int n = 0;
        ForkJoinPool.WorkQueue[] workQueues;
        if ((workQueues = this.workQueues) != null) {
            for (int i = 0; i < workQueues.length; i += 2) {
                ForkJoinPool.WorkQueue workQueue;
                if ((workQueue = workQueues[i]) != null) {
                    n += workQueue.queueSize();
                }
            }
        }
        return n;
    }
    
    public boolean hasQueuedSubmissions() {
        ForkJoinPool.WorkQueue[] workQueues;
        if ((workQueues = this.workQueues) != null) {
            int i = 0;
            while (i < workQueues.length) {
                ForkJoinPool.WorkQueue workQueue;
                if ((workQueue = workQueues[i]) != null && !workQueue.isEmpty()) {
                    return true;
                }
                i += 2;
            }
        }
        return false;
    }
    
    protected ForkJoinTask<?> pollSubmission() {
        ForkJoinPool.WorkQueue[] workQueues;
        if ((workQueues = this.workQueues) != null) {
            int i = 0;
            while (i < workQueues.length) {
                ForkJoinPool.WorkQueue workQueue;
                ForkJoinTask<Object> poll;
                if ((workQueue = workQueues[i]) != null && (poll = workQueue.poll()) != null) {
                    return poll;
                }
                i += 2;
            }
        }
        return null;
    }
    
    protected int drainTasksTo(Collection<? super ForkJoinTask<?>> collection) {
        int n = 0;
        ForkJoinPool.WorkQueue[] workQueues;
        if ((workQueues = this.workQueues) != null) {
            for (int i = 0; i < workQueues.length; ++i) {
                ForkJoinPool.WorkQueue workQueue;
                if ((workQueue = workQueues[i]) != null) {
                    ForkJoinTask<Object> poll;
                    while ((poll = workQueue.poll()) != null) {
                        collection.add(poll);
                        ++n;
                    }
                }
            }
        }
        return n;
    }
    
    @Override
    public String toString() {
        long lng = 0L;
        long lng2 = 0L;
        int i = 0;
        AtomicLong stealCounter = this.stealCounter;
        long lng3 = (stealCounter == null) ? 0L : stealCounter.get();
        long ctl = this.ctl;
        ForkJoinPool.WorkQueue[] workQueues;
        if ((workQueues = this.workQueues) != null) {
            for (int j = 0; j < workQueues.length; ++j) {
                ForkJoinPool.WorkQueue workQueue;
                if ((workQueue = workQueues[j]) != null) {
                    int queueSize = workQueue.queueSize();
                    if ((j & 0x1) == 0x0) {
                        lng2 += (long)queueSize;
                    }
                    else {
                        lng += (long)queueSize;
                        lng3 += (long)workQueue.nsteals;
                        if (workQueue.isApparentlyUnblocked()) {
                            ++i;
                        }
                    }
                }
            }
        }
        int k = this.config & 0xFFFF;
        int l = k + (short)(int)(ctl >>> 32);
        int m = k + (int)(ctl >> 48);
        if (m < 0) {
            m = 0;
        }
        int runState = this.runState;
        return super.toString() + "[" + (((runState & 0x40000000) != 0x0) ? "Terminated" : (((runState & 0x20000000) != 0x0) ? "Terminating" : (((runState & Integer.MIN_VALUE) != 0x0) ? "Shutting down" : "Running"))) + ", parallelism = " + k + ", size = " + l + ", active = " + m + ", running = " + i + ", steals = " + lng3 + ", tasks = " + lng + ", submissions = " + lng2 + "]";
    }
    
    @Override
    public void shutdown() {
        checkPermission();
        this.tryTerminate(false, true);
    }
    
    @Override
    public List<Runnable> shutdownNow() {
        checkPermission();
        this.tryTerminate(true, true);
        return Collections.emptyList();
    }
    
    @Override
    public boolean isTerminated() {
        return (this.runState & 0x40000000) != 0x0;
    }
    
    public boolean isTerminating() {
        int runState = this.runState;
        return (runState & 0x20000000) != 0x0 && (runState & 0x40000000) == 0x0;
    }
    
    @Override
    public boolean isShutdown() {
        return (this.runState & Integer.MIN_VALUE) != 0x0;
    }
    
    @Override
    public boolean awaitTermination(long n, TimeUnit timeUnit) throws InterruptedException {
        if (Thread.interrupted()) {
            throw new InterruptedException();
        }
        if (this == ForkJoinPool.common) {
            this.awaitQuiescence(n, timeUnit);
            return false;
        }
        long nanos = timeUnit.toNanos(n);
        if (this.isTerminated()) {
            return true;
        }
        if (nanos <= 0L) {
            return false;
        }
        long n2 = System.nanoTime() + nanos;
        synchronized (this) {
            while (!this.isTerminated()) {
                if (nanos <= 0L) {
                    return false;
                }
                long millis = TimeUnit.NANOSECONDS.toMillis(nanos);
                this.wait((millis > 0L) ? millis : 1L);
                nanos = n2 - System.nanoTime();
            }
            return true;
        }
    }
    
    public boolean awaitQuiescence(long n, TimeUnit timeUnit) {
        long nanos = timeUnit.toNanos(n);
        Thread currentThread = Thread.currentThread();
        ForkJoinWorkerThread forkJoinWorkerThread;
        if (currentThread instanceof ForkJoinWorkerThread && (forkJoinWorkerThread = (ForkJoinWorkerThread)currentThread).pool == this) {
            this.helpQuiescePool(forkJoinWorkerThread.workQueue);
            return true;
        }
        long nanoTime = System.nanoTime();
        int n2 = 0;
        int n3 = 1;
        ForkJoinPool.WorkQueue[] workQueues;
        int n4;
        while (!this.isQuiescent() && (workQueues = this.workQueues) != null && (n4 = workQueues.length - 1) >= 0) {
            if (n3 == 0) {
                if (System.nanoTime() - nanoTime > nanos) {
                    return false;
                }
                Thread.yield();
            }
            n3 = 0;
            int i = n4 + 1 << 2;
            while (i >= 0) {
                int n5;
                ForkJoinPool.WorkQueue workQueue;
                int base;
                if ((n5 = (n2++ & n4)) <= n4 && n5 >= 0 && (workQueue = workQueues[n5]) != null && (base = workQueue.base) - workQueue.top < 0) {
                    n3 = 1;
                    ForkJoinTask<Object> poll;
                    if ((poll = workQueue.pollAt(base)) != null) {
                        poll.doExec();
                        break;
                    }
                    break;
                }
                else {
                    --i;
                }
            }
        }
        return true;
    }
    
    static void quiesceCommonPool() {
        ForkJoinPool.common.awaitQuiescence(Long.MAX_VALUE, TimeUnit.NANOSECONDS);
    }
    
    public static void managedBlock(ForkJoinPool.ManagedBlocker managedBlocker) throws InterruptedException {
        Thread currentThread = Thread.currentThread();
        ForkJoinWorkerThread forkJoinWorkerThread;
        ForkJoinPool pool;
        if (currentThread instanceof ForkJoinWorkerThread && (pool = (forkJoinWorkerThread = (ForkJoinWorkerThread)currentThread).pool) != null) {
            ForkJoinPool.WorkQueue workQueue = forkJoinWorkerThread.workQueue;
            while (!managedBlocker.isReleasable()) {
                if (!pool.tryCompensate(workQueue)) {
                    continue;
                }
                try {
                    while (!managedBlocker.isReleasable() && !managedBlocker.block()) {}
                }
                finally {
                    ForkJoinPool.U.getAndAddLong(pool, ForkJoinPool.CTL, 281474976710656L);
                }
            }
        }
        else {
            while (!managedBlocker.isReleasable() && !managedBlocker.block()) {}
        }
    }
    
    @Override
    protected <T> RunnableFuture<T> newTaskFor(Runnable runnable, T t) {
        return new ForkJoinTask.AdaptedRunnable(runnable, t);
    }
    
    @Override
    protected <T> RunnableFuture<T> newTaskFor(Callable<T> callable) {
        return new ForkJoinTask.AdaptedCallable(callable);
    }
    
    private static ForkJoinPool makeCommonPool() {
        ForkJoinPool.CommonPoolForkJoinWorkerThreadFactory commonPoolForkJoinWorkerThreadFactory = new ForkJoinPool.CommonPoolForkJoinWorkerThreadFactory();
        int int1 = -1;
        commonPoolForkJoinWorkerThreadFactory = null;
        Thread.UncaughtExceptionHandler uncaughtExceptionHandler = null;
        try {
            String property = System.getProperty("java.util.concurrent.ForkJoinPool.common.parallelism");
            String property2 = System.getProperty("java.util.concurrent.ForkJoinPool.common.threadFactory");
            String property3 = System.getProperty("java.util.concurrent.ForkJoinPool.common.exceptionHandler");
            if (property != null) {
                int1 = Integer.parseInt(property);
            }
            if (property2 != null) {
                commonPoolForkJoinWorkerThreadFactory = (ForkJoinPool.ForkJoinWorkerThreadFactory)ClassLoader.getSystemClassLoader().loadClass(property2).newInstance();
            }
            if (property3 != null) {
                uncaughtExceptionHandler = (Thread.UncaughtExceptionHandler)ClassLoader.getSystemClassLoader().loadClass(property3).newInstance();
            }
        }
        catch (Exception ex) {}
        if (commonPoolForkJoinWorkerThreadFactory == null) {
            if (System.getSecurityManager() == null) {
            }
            else {
                commonPoolForkJoinWorkerThreadFactory = new ForkJoinPool.InnocuousForkJoinWorkerThreadFactory();
            }
        }
        if (int1 < 0 && (int1 = Runtime.getRuntime().availableProcessors() - 1) <= 0) {
            int1 = 1;
        }
        if (int1 > 32767) {
            int1 = 32767;
        }
        return new ForkJoinPool(int1, commonPoolForkJoinWorkerThreadFactory, uncaughtExceptionHandler, 0, "ForkJoinPool.commonPool-worker-");
    }
    
    static ForkJoinPool access$000() {
        return makeCommonPool();
    }
    
    static {
        try {
            U = Unsafe.getUnsafe();
            Class<Object> clazz = ForkJoinPool.class;
            CTL = ForkJoinPool.U.objectFieldOffset(clazz.getDeclaredField("ctl"));
            RUNSTATE = ForkJoinPool.U.objectFieldOffset(clazz.getDeclaredField("runState"));
            STEALCOUNTER = ForkJoinPool.U.objectFieldOffset(clazz.getDeclaredField("stealCounter"));
            PARKBLOCKER = ForkJoinPool.U.objectFieldOffset(Thread.class.getDeclaredField("parkBlocker"));
            Class<Object> clazz2 = ForkJoinPool.WorkQueue.class;
            QTOP = ForkJoinPool.U.objectFieldOffset(clazz2.getDeclaredField("top"));
            QLOCK = ForkJoinPool.U.objectFieldOffset(clazz2.getDeclaredField("qlock"));
            QSCANSTATE = ForkJoinPool.U.objectFieldOffset(clazz2.getDeclaredField("scanState"));
            QPARKER = ForkJoinPool.U.objectFieldOffset(clazz2.getDeclaredField("parker"));
            QCURRENTSTEAL = ForkJoinPool.U.objectFieldOffset(clazz2.getDeclaredField("currentSteal"));
            QCURRENTJOIN = ForkJoinPool.U.objectFieldOffset(clazz2.getDeclaredField("currentJoin"));
            Class<Object> clazz3 = ForkJoinTask[].class;
            ABASE = ForkJoinPool.U.arrayBaseOffset(clazz3);
            int arrayIndexScale = ForkJoinPool.U.arrayIndexScale(clazz3);
            if ((arrayIndexScale & arrayIndexScale - 1) != 0x0) {
                throw new Error("data type scale not a power of two");
            }
            ASHIFT = 31 - Integer.numberOfLeadingZeros(arrayIndexScale);
        }
        catch (Exception cause) {
            throw new Error(cause);
        }
        ForkJoinPool.commonMaxSpares = 256;
        defaultForkJoinWorkerThreadFactory = new ForkJoinPool.DefaultForkJoinWorkerThreadFactory();
        modifyThreadPermission = new RuntimePermission("modifyThread");
        common = (ForkJoinPool)AccessController.doPrivileged(new PrivilegedAction<ForkJoinPool>() {
            @Override
            public ForkJoinPool run() {
                return ForkJoinPool.access$000();
            }
        });
        int n = ForkJoinPool.common.config & 0xFFFF;
        commonParallelism = ((n > 0) ? n : true);
    }
    
    static final class DefaultForkJoinWorkerThreadFactory implements ForkJoinPool.ForkJoinWorkerThreadFactory
    {
        @Override
        public final ForkJoinWorkerThread newThread(ForkJoinPool forkJoinPool) {
            return new ForkJoinWorkerThread(forkJoinPool);
        }
    }
    
    static final class CommonPoolForkJoinWorkerThreadFactory implements ForkJoinPool.ForkJoinWorkerThreadFactory
    {
        @Override
        public final ForkJoinWorkerThread newThread(ForkJoinPool forkJoinPool) {
            return new ForkJoinWorkerThread(forkJoinPool, null);
        }
    }
    
    static final class EmptyTask extends ForkJoinTask<Void>
    {
        private static final long serialVersionUID = -7721805057305804111L;
        
        EmptyTask() {
            this.status = -268435456;
        }
        
        @Override
        public final Void getRawResult() {
            return null;
        }
        
        public final void setRawResult(Void void1) {
        }
        
        public final boolean exec() {
            return true;
        }
    }
    
    @Contended
    static final class WorkQueue
    {
        static final int INITIAL_QUEUE_CAPACITY = 8192;
        static final int MAXIMUM_QUEUE_CAPACITY = 67108864;
        volatile int scanState;
        int stackPred;
        int nsteals;
        int hint;
        int config;
        volatile int qlock;
        volatile int base;
        int top;
        ForkJoinTask<?>[] array;
        final ForkJoinPool pool;
        final ForkJoinWorkerThread owner;
        volatile Thread parker;
        volatile ForkJoinTask<?> currentJoin;
        volatile ForkJoinTask<?> currentSteal;
        private static final Unsafe U;
        private static final int ABASE;
        private static final int ASHIFT;
        private static final long QTOP;
        private static final long QLOCK;
        private static final long QCURRENTSTEAL;
        
        WorkQueue(ForkJoinPool pool, ForkJoinWorkerThread owner) {
            this.pool = pool;
            this.owner = owner;
            int n = 4096;
            this.top = n;
            this.base = n;
        }
        
        final int getPoolIndex() {
            return (this.config & 0xFFFF) >>> 1;
        }
        
        final int queueSize() {
            int n = this.base - this.top;
            return (n >= 0) ? 0 : (-n);
        }
        
        final boolean isEmpty() {
            int top;
            int n;
            ForkJoinTask[] array;
            int n2;
            return (n = this.base - (top = this.top)) >= 0 || (n == -1 && ((array = this.array) == null || (n2 = array.length - 1) < 0 || ForkJoinPool.WorkQueue.U.getObject(array, (long)((n2 & top - 1) << ForkJoinPool.WorkQueue.ASHIFT) + (long)ForkJoinPool.WorkQueue.ABASE) == null));
        }
        
        final void push(ForkJoinTask<?> x) {
            int base = this.base;
            int top = this.top;
            ForkJoinTask[] array;
            if ((array = this.array) != null) {
                int n = array.length - 1;
                ForkJoinPool.WorkQueue.U.putOrderedObject(array, (long)(((n & top) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE), x);
                ForkJoinPool.WorkQueue.U.putOrderedInt(this, ForkJoinPool.WorkQueue.QTOP, top + 1);
                int n2;
                if ((n2 = top - base) <= 1) {
                    ForkJoinPool pool;
                    if ((pool = this.pool) != null) {
                        pool.signalWork(pool.workQueues, this);
                    }
                }
                else if (n2 >= n) {
                    this.growArray();
                }
            }
        }
        
        final ForkJoinTask<?>[] growArray() {
            ForkJoinTask[] array = this.array;
            int n = (array != null) ? (array.length << 1) : 8192;
            if (n > 67108864) {
                throw new RejectedExecutionException("Queue capacity exceeded");
            }
            ForkJoinTask[] array2 = new ForkJoinTask[n];
            this.array = array2;
            ForkJoinTask[] o = array2;
            int n2;
            if (array != null && (n2 = array.length - 1) >= 0) {
                int top = this.top;
                int base;
                if (top - (base = this.base) > 0) {
                    int n3 = n - 1;
                    do {
                        int n4 = ((base & n2) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE;
                        int n5 = ((base & n3) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE;
                        ForkJoinTask<Object> forkJoinTask = (ForkJoinTask)ForkJoinPool.WorkQueue.U.getObjectVolatile(array, (long)n4);
                        if (forkJoinTask != null && ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, (long)n4, forkJoinTask, null)) {
                            ForkJoinPool.WorkQueue.U.putObjectVolatile(o, (long)n5, forkJoinTask);
                        }
                        else {
                            continue;
                        }
                    } while (++base != top);
                }
            }
            return o;
        }
        
        final ForkJoinTask<?> pop() {
            ForkJoinTask[] array;
            int n;
            if ((array = this.array) != null && (n = array.length - 1) >= 0) {
                int x;
                while ((x = this.top - 1) - this.base >= 0) {
                    long n2 = (long)(((n & x) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE);
                    ForkJoinTask<Object> expected;
                    if ((expected = (ForkJoinTask)ForkJoinPool.WorkQueue.U.getObject(array, n2)) == null) {
                        break;
                    }
                    if (ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, n2, expected, null)) {
                        ForkJoinPool.WorkQueue.U.putOrderedInt(this, ForkJoinPool.WorkQueue.QTOP, x);
                        return expected;
                    }
                    continue;
                }
            }
            return null;
        }
        
        final ForkJoinTask<?> pollAt(int n) {
            ForkJoinTask[] array;
            if ((array = this.array) != null) {
                int n2 = ((array.length - 1 & n) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE;
                ForkJoinTask<Object> expected;
                if ((expected = (ForkJoinTask)ForkJoinPool.WorkQueue.U.getObjectVolatile(array, (long)n2)) != null && this.base == n && ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, (long)n2, expected, null)) {
                    this.base = n + 1;
                    return expected;
                }
            }
            return null;
        }
        
        final ForkJoinTask<?> poll() {
            int base;
            ForkJoinTask[] array;
            while ((base = this.base) - this.top < 0 && (array = this.array) != null) {
                int n = ((array.length - 1 & base) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE;
                ForkJoinTask<Object> expected = (ForkJoinTask)ForkJoinPool.WorkQueue.U.getObjectVolatile(array, (long)n);
                if (this.base == base) {
                    if (expected != null) {
                        if (ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, (long)n, expected, null)) {
                            this.base = base + 1;
                            return expected;
                        }
                        continue;
                    }
                    else {
                        if (base + 1 == this.top) {
                            break;
                        }
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
            return null;
        }
        
        final ForkJoinTask<?> nextLocalTask() {
            return ((this.config & 0x10000) == 0x0) ? this.pop() : this.poll();
        }
        
        final ForkJoinTask<?> peek() {
            ForkJoinTask[] array = this.array;
            int n;
            if (array == null || (n = array.length - 1) < 0) {
                return null;
            }
            return (ForkJoinTask)ForkJoinPool.WorkQueue.U.getObjectVolatile(array, (long)((((((this.config & 0x10000) == 0x0) ? (this.top - 1) : this.base) & n) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE));
        }
        
        final boolean tryUnpush(ForkJoinTask<?> expected) {
            ForkJoinTask[] array;
            int top;
            if ((array = this.array) != null && (top = this.top) != this.base && ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, (long)(((array.length - 1 & --top) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE), expected, null)) {
                ForkJoinPool.WorkQueue.U.putOrderedInt(this, ForkJoinPool.WorkQueue.QTOP, top);
                return true;
            }
            return false;
        }
        
        final void cancelAll() {
            ForkJoinTask<Object> currentJoin;
            if ((currentJoin = this.currentJoin) != null) {
                this.currentJoin = null;
                ForkJoinTask.cancelIgnoringExceptions(currentJoin);
            }
            ForkJoinTask<Object> currentSteal;
            if ((currentSteal = this.currentSteal) != null) {
                this.currentSteal = null;
                ForkJoinTask.cancelIgnoringExceptions(currentSteal);
            }
            ForkJoinTask<Object> poll;
            while ((poll = this.poll()) != null) {
                ForkJoinTask.cancelIgnoringExceptions(poll);
            }
        }
        
        final void pollAndExecAll() {
            ForkJoinTask<Object> poll;
            while ((poll = this.poll()) != null) {
                poll.doExec();
            }
        }
        
        final void execLocalTasks() {
            int base = this.base;
            ForkJoinTask[] array = this.array;
            int x;
            int n;
            if (base - (x = this.top - 1) <= 0 && array != null && (n = array.length - 1) >= 0) {
                if ((this.config & 0x10000) == 0x0) {
                    ForkJoinTask<Object> forkJoinTask;
                    while ((forkJoinTask = (ForkJoinTask)ForkJoinPool.WorkQueue.U.getAndSetObject(array, (long)(((n & x) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE), null)) != null) {
                        ForkJoinPool.WorkQueue.U.putOrderedInt(this, ForkJoinPool.WorkQueue.QTOP, x);
                        forkJoinTask.doExec();
                        if (this.base - (x = this.top - 1) <= 0) {
                            continue;
                        }
                        return;
                    }
                    return;
                }
                this.pollAndExecAll();
            }
        }
        
        final void runTask(ForkJoinTask<?> currentSteal) {
            if (currentSteal != null) {
                this.scanState &= 0xFFFFFFFE;
                (this.currentSteal = currentSteal).doExec();
                ForkJoinPool.WorkQueue.U.putOrderedObject(this, ForkJoinPool.WorkQueue.QCURRENTSTEAL, null);
                this.execLocalTasks();
                ForkJoinWorkerThread owner = this.owner;
                if (++this.nsteals < 0) {
                    this.transferStealCount(this.pool);
                }
                this.scanState |= 0x1;
                if (owner != null) {
                    owner.afterTopLevelExec();
                }
            }
        }
        
        final void transferStealCount(ForkJoinPool forkJoinPool) {
            AtomicLong stealCounter;
            if (forkJoinPool != null && (stealCounter = forkJoinPool.stealCounter) != null) {
                int nsteals = this.nsteals;
                this.nsteals = 0;
                stealCounter.getAndAdd((long)((nsteals < 0) ? Integer.MAX_VALUE : nsteals));
            }
        }
        
        final boolean tryRemoveAndExec(ForkJoinTask<?> forkJoinTask) {
            ForkJoinTask[] array;
            int n;
            if ((array = this.array) != null && (n = array.length - 1) >= 0 && forkJoinTask != null) {
                int top;
                int base;
                int n2;
                while ((n2 = (top = this.top) - (base = this.base)) > 0) {
                    while (true) {
                        long n3 = (long)(((--top & n) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE);
                        ForkJoinTask<Object> expected;
                        if ((expected = (ForkJoinTask)ForkJoinPool.WorkQueue.U.getObject(array, n3)) == null) {
                            return top + 1 == this.top;
                        }
                        if (expected == forkJoinTask) {
                            boolean compareAndSwapObject = false;
                            if (top + 1 == this.top) {
                                if (ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, n3, forkJoinTask, null)) {
                                    ForkJoinPool.WorkQueue.U.putOrderedInt(this, ForkJoinPool.WorkQueue.QTOP, top);
                                    compareAndSwapObject = true;
                                }
                            }
                            else if (this.base == base) {
                                compareAndSwapObject = ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, n3, forkJoinTask, new ForkJoinPool.EmptyTask());
                            }
                            if (!(compareAndSwapObject == 0)) {
                                forkJoinTask.doExec();
                                break;
                            }
                            break;
                        }
                        else if (expected.status < 0 && top + 1 == this.top) {
                            if (ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, n3, expected, null)) {
                                ForkJoinPool.WorkQueue.U.putOrderedInt(this, ForkJoinPool.WorkQueue.QTOP, top);
                                break;
                            }
                            break;
                        }
                        else {
                            if (--n2 == 0) {
                                return false;
                            }
                            continue;
                        }
                    }
                    if (forkJoinTask.status >= 0) {
                        continue;
                    }
                    return false;
                }
            }
            return true;
        }
        
        final CountedCompleter<?> popCC(CountedCompleter<?> countedCompleter, int n) {
            int top;
            ForkJoinTask[] array;
            if (this.base - (top = this.top) < 0 && (array = this.array) != null) {
                long offset = (long)(((array.length - 1 & top - 1) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE);
                Object objectVolatile;
                if ((objectVolatile = ForkJoinPool.WorkQueue.U.getObjectVolatile(array, offset)) != null && objectVolatile instanceof CountedCompleter) {
                    CountedCompleter<Object> completer = (CountedCompleter)objectVolatile;
                    while (completer != countedCompleter) {
                        if ((completer = completer.completer) == null) {
                            return null;
                        }
                        continue;
                    }
                    if (n < 0) {
                        if (ForkJoinPool.WorkQueue.U.compareAndSwapInt(this, ForkJoinPool.WorkQueue.QLOCK, 0, 1)) {
                            if (this.top == top && this.array == array && ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, offset, completer, null)) {
                                ForkJoinPool.WorkQueue.U.putOrderedInt(this, ForkJoinPool.WorkQueue.QTOP, top - 1);
                                ForkJoinPool.WorkQueue.U.putOrderedInt(this, ForkJoinPool.WorkQueue.QLOCK, 0);
                                return completer;
                            }
                            ForkJoinPool.WorkQueue.U.compareAndSwapInt(this, ForkJoinPool.WorkQueue.QLOCK, 1, 0);
                        }
                    }
                    else if (ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, offset, completer, null)) {
                        ForkJoinPool.WorkQueue.U.putOrderedInt(this, ForkJoinPool.WorkQueue.QTOP, top - 1);
                        return completer;
                    }
                }
            }
            return null;
        }
        
        final int pollAndExecCC(CountedCompleter<?> countedCompleter) {
            int base;
            ForkJoinTask[] array;
            int n2;
            if ((base = this.base) - this.top >= 0 || (array = this.array) == null) {
                n2 = (base | Integer.MIN_VALUE);
            }
            else {
                long n = (long)(((array.length - 1 & base) << ForkJoinPool.WorkQueue.ASHIFT) + ForkJoinPool.WorkQueue.ABASE);
                Object objectVolatile;
                if ((objectVolatile = ForkJoinPool.WorkQueue.U.getObjectVolatile(array, n)) == null) {
                    n2 = 2;
                }
                else if (!(objectVolatile instanceof CountedCompleter)) {
                    n2 = -1;
                }
                else {
                    CountedCompleter<Object> completer = (CountedCompleter)objectVolatile;
                    while (completer != countedCompleter) {
                        if ((completer = completer.completer) != null) {
                            continue;
                        }
                        n2 = -1;
                        return n2;
                    }
                    if (this.base == base && ForkJoinPool.WorkQueue.U.compareAndSwapObject(array, n, completer, null)) {
                        this.base = base + 1;
                        completer.doExec();
                        n2 = 1;
                    }
                    else {
                        n2 = 2;
                    }
                }
            }
            return n2;
        }
        
        final boolean isApparentlyUnblocked() {
            ForkJoinWorkerThread owner;
            Thread.State state;
            return this.scanState >= 0 && (owner = this.owner) != null && (state = owner.getState()) != Thread.State.BLOCKED && state != Thread.State.WAITING && state != Thread.State.TIMED_WAITING;
        }
        
        static {
            try {
                U = Unsafe.getUnsafe();
                Class<Object> clazz = ForkJoinPool.WorkQueue.class;
                Class<Object> clazz2 = ForkJoinTask[].class;
                QTOP = ForkJoinPool.WorkQueue.U.objectFieldOffset(clazz.getDeclaredField("top"));
                QLOCK = ForkJoinPool.WorkQueue.U.objectFieldOffset(clazz.getDeclaredField("qlock"));
                QCURRENTSTEAL = ForkJoinPool.WorkQueue.U.objectFieldOffset(clazz.getDeclaredField("currentSteal"));
                ABASE = ForkJoinPool.WorkQueue.U.arrayBaseOffset(clazz2);
                int arrayIndexScale = ForkJoinPool.WorkQueue.U.arrayIndexScale(clazz2);
                if ((arrayIndexScale & arrayIndexScale - 1) != 0x0) {
                    throw new Error("data type scale not a power of two");
                }
                ASHIFT = 31 - Integer.numberOfLeadingZeros(arrayIndexScale);
            }
            catch (Exception cause) {
                throw new Error(cause);
            }
        }
    }
    
    static final class InnocuousForkJoinWorkerThreadFactory implements ForkJoinPool.ForkJoinWorkerThreadFactory
    {
        private static final AccessControlContext innocuousAcc;
        
        @Override
        public final ForkJoinWorkerThread newThread(ForkJoinPool forkJoinPool) {
            return (ForkJoinWorkerThread.InnocuousForkJoinWorkerThread)AccessController.doPrivileged(new PrivilegedAction<ForkJoinWorkerThread>() {
                @Override
                public ForkJoinWorkerThread run() {
                    return new ForkJoinWorkerThread.InnocuousForkJoinWorkerThread(forkJoinPool);
                }
            }, ForkJoinPool.InnocuousForkJoinWorkerThreadFactory.innocuousAcc);
        }
        
        static {
            Permissions permissions = new Permissions();
            permissions.add(ForkJoinPool.modifyThreadPermission);
            permissions.add(new RuntimePermission("enableContextClassLoaderOverride"));
            permissions.add(new RuntimePermission("modifyThreadGroup"));
            innocuousAcc = new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(null, permissions) });
        }
    }
    
    public interface ForkJoinWorkerThreadFactory
    {
        public abstract ForkJoinWorkerThread newThread(ForkJoinPool p0);
    }
    
    public interface ManagedBlocker
    {
        public abstract boolean block() throws InterruptedException;
        
        public abstract boolean isReleasable();
    }
}
